/**********************************************************************
  PROJECT: DD Animation Demo
  Source: DD_Anim.cpp
  Chapter reference: 12
***********************************************************************
  Description: 
     DirectDraw animation in a window.

  Topics:
  1. Using a progressive image set in animation
  2. Modifying the message loop for timed blitting
  3. Obtaining the animation time pulse (tick counter method)
  4. Blitting a bitmap image section
**********************************************************************/ 

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN  // Exclude unnecessary window.h components
#endif

#include <windows.h>
#include <ddraw.h>
#include <stdio.h>
#include <stdarg.h>

//Global variables
LPDIRECTDRAW4				lpDD4 = NULL;        // DirectDraw object
LPDIRECTDRAWSURFACE4        lpDDSPrimary = NULL; // DirectDraw primary
LPDIRECTDRAWSURFACE4		lpDDSOffscreen = NULL; 
DDSURFACEDESC2              ddsd; 
HWND               hWnd;
HRESULT            hRet;
RECT			   clientArea;          // For Blt() destination
RECT			   bmapArea;            // For Blt() source
int 			   frameNum = 0;        // Frame number
int			       apOn = 0;
DWORD			   thisTickCount;
DWORD			   lastTickCount;
static DWORD       TIMER_VALUE = 10;


// Constants idenfying bitmap image set
static char bmapName[] = {"gearsy.bmp"}; 
static int				 imageCols = 6;      // Number of image columns
static int				 imageRows = 3;      // Number of rows

// Variables, constants, handles, and structure for bitmaps
int         frameCount = (imageCols * imageRows) - 1;
int         bmapXSize;			// x size of bitmap
int         bmapYSize;          // y size of bitmap
HBITMAP		aBitmap;
BITMAP		bMap1;          // Structures for bitmap data

//*************************************
//            Prototypes
//*************************************
void ReleaseObjects(void);
HRESULT DDInitFailed(HWND, HRESULT, LPCTSTR);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
int DD4Interface();
HBITMAP DDLoadBitmap( LPDIRECTDRAWSURFACE4, LPSTR );
BOOL DDShowBitmap( LPDIRECTDRAWSURFACE4, HBITMAP,
				   int, int, int, int);
HRESULT DDBmapToSurf(LPDIRECTDRAWSURFACE4, HBITMAP);
static void BlitSprite();


//*****************************************************************
// WinMain implementation for exclusive mode DirectDraw application
//*****************************************************************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{

    MSG             msg;
	RECT			progWin;   // Application window dimensions


    static char szAppName[] = "DDAppName" ; // Application name

     // Defining a structure of type WNDCLASSEX
     WNDCLASSEX  wndclass ;
     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = NULL;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject
                              (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName;
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;
	 
	 // Register the class
	 RegisterClassEx(&wndclass);


	//**************************************
	//     Load bitmap into memory
    //**************************************
	// Load the bitmap image into memory
	aBitmap = ( HBITMAP )LoadImage( NULL, bmapName,
            IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE );
    if ( aBitmap == NULL ) 
         DDInitFailed(hWnd, hRet,
		      "Bitmap load failed in DDLoadBitmap()");

	// Get bitmap data
	GetObject(aBitmap, sizeof (BITMAP), &bMap1);

	// Calculate and store bitmap and image data
	bmapXSize = bMap1.bmWidth / imageCols;
	bmapYSize = bMap1.bmHeight / imageRows;

	// Store bitmap in RECT structure variable
    progWin.left = 0;
	progWin.top = 0;
	progWin.right = bmapXSize;
	progWin.bottom = bmapYSize;

	//*****************************************
	//   Create window window with client 
	//   area the same size as the bitmap
	//*****************************************
    // First adjust the size of the client area to the size
	// of the bounding rectangle (this includes the border,
	// caption bar, menu, etc.)

	 AdjustWindowRectEx(&progWin,
                       WS_SYSMENU | WS_CAPTION,
					   FALSE,
					   0);
              
     hWnd = CreateWindowEx(0,        // Extended style
                szAppName,
                "DirectDraw Animation Demo",
				WS_SYSMENU | WS_CAPTION,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				(progWin.right - progWin.left),
				(progWin.bottom - progWin.top),
				NULL,            // Handle of parent
                NULL,            // Handle to menu
                hInstance,       // Application instance
                NULL);           // Additional data

    if (!hWnd)
        return FALSE;

    ShowWindow(hWnd, nCmdShow);


	//*************************************
	// Create DirectDraw object and 
	// create primary surface
	//*************************************
    // Fetch DirectDraw4 interface
	hRet = DD4Interface();
	if (hRet == 0)
		return DDInitFailed(hWnd, hRet,
		                    "QueryInterface() call failed");

    // Set cooperative level to exclusive and full screen
    hRet = lpDD4->SetCooperativeLevel(hWnd, DDSCL_NORMAL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetCooperativeLevel() call failed");

    //**********************************
	//    Create the primary surface
	//**********************************
	// ddsd is a structure of type DDSRUFACEDESC2
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS ;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

    hRet = lpDD4->CreateSurface(&ddsd, &lpDDSPrimary, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "CreateSurface() call failed");

	//**************************************
	//   Store bitmap in off screen surface
    //**************************************
    // First create an off-screen surface
	// in system memory
	ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
    ddsd.dwHeight = bMap1.bmHeight;
	ddsd.dwWidth = bMap1.bmWidth;
    hRet = lpDD4->CreateSurface(&ddsd, &lpDDSOffscreen, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		         "Off Screen surface creation failed");
     // Move bitmap to surface using DDBmapToSurf()function 
	 hRet = DDBmapToSurf(lpDDSOffscreen, aBitmap);
     if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "DDBMapToSurf() call failed");
	 // ASSERT:
	 //     Bitmap is in offscreen surface -> lpDDSOffscreen

	 //***********************************
	 //      Blit the bitmap
	 //***********************************
	 BlitSprite();
     apOn = 1;

    while ( 1 )
    {
        if ( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if ( !GetMessage( &msg, NULL, 0, 0 ) )
            {
                return msg.wParam;
            }
            TranslateMessage( &msg );
            DispatchMessage( &msg );
        }
        else if ( apOn == 1 )  
           BlitSprite();
        
        else
           WaitMessage();
     }
}

//*****************************************************************
// Name: WndProc
// Desc: Window procedure
//*****************************************************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam,
LPARAM lParam)
{


    switch (message)
    {

        case WM_KEYDOWN:
            // Handle keystroke to end execution
            switch (wParam)
            {
                case VK_ESCAPE:
                    PostMessage(hwnd, WM_CLOSE, 0, 0);
                    return 0L;
			}
			break;

		case WM_MOVE:
			GetClientRect(hWnd, &clientArea);
			ClientToScreen(hWnd, (LPPOINT) &clientArea.left);
			ClientToScreen(hWnd, (LPPOINT) &clientArea.right);
			return 0L;
			break;
 
         
        case WM_DESTROY:
            // Clean up and close the app
            ReleaseObjects();
            PostQuitMessage(0);
            return 0L;
			break;

	}
    return DefWindowProc(hwnd, message, wParam, lParam);
}

//**************************************
//      update animation frame
//**************************************
static void BlitSprite()
{


   thisTickCount = GetTickCount();

   if((thisTickCount - lastTickCount) < TIMER_VALUE)
	   return;
   else
   {
		// Update the sprite image with the current frame.
		bmapArea.top = ( ( frameNum / imageCols) * bmapYSize );
		bmapArea.left = ( ( frameNum % imageCols) * bmapXSize );
		bmapArea.bottom = bmapArea.top + bmapYSize;
		bmapArea.right = bmapArea.left + bmapXSize;

	     hRet = lpDDSPrimary->Blt( &clientArea, lpDDSOffscreen,
			    &bmapArea, DDBLT_WAIT, NULL );
			if(hRet != DD_OK)
				DDInitFailed(hWnd, hRet, "Blt() failed");

		// Update the frame counter
		frameNum++;
		if(frameNum > frameCount)
			frameNum = 0;
   lastTickCount = thisTickCount;
   return;
   }
}

//************************************
// Name: ReleaseObject
// Desc: Releases all objects
//************************************
static void ReleaseObjects(void)
{
    if (lpDD4 != NULL)
    {
        if (lpDDSPrimary != NULL)
        {
            lpDDSPrimary->Release();
            lpDDSPrimary = NULL;
        }
        lpDD4->Release();
        lpDD4 = NULL;
    }
}

//***************************************
// Name: DDInitFailed()
// Desc: This function is called if an
//       initialization call fails
//***************************************
HRESULT DDInitFailed(HWND hWnd, HRESULT hRet, LPCTSTR szError)
{
    char   szBuff[128];

    sprintf(szBuff, szError);
    ReleaseObjects();
    MessageBox(hWnd, szBuff, "DirectDraw InWin Demo", MB_OK);
    DestroyWindow(hWnd);
    return hRet;

}

//******************************************
// Name: DD4Interface()
// Desc: Finds DirectDraw4 object
// 
// POST CONDITIONS:
//   returns 0 if no DirectDraw4 interface
//   returns 1 if DirectDraw4 found
//******************************************
int DD4Interface()
{

	HRESULT         DDConnect;
	LPDIRECTDRAW	lpDD;    // Pointer to DirectDraw

	DDConnect = DirectDrawCreate ( NULL,
	           &lpDD,
			   NULL);
     
	if(DDConnect != DD_OK)
		return 0;
    // Atempt to locate DirectDraw4 interface
    DDConnect = lpDD->QueryInterface(
                IID_IDirectDraw4,
	            (LPVOID *) &lpDD4);
    if(DDConnect != S_OK)
		return 0;

    lpDD->Release();      // Release old pointer
        return 1;

}

//***************************************************************
// Name: DDShowBitmap
// Desc: Displays a bitmap using a DirectDraw device context
//
// PRE CONDITIONS:
// 1. Parameter 1 is pointer to a DirectDraw surface
//    Parameter 2 is handle to the bitmap
//    Parameters 3 and 4 are the display location
//    Parameters 5 and 6 are the bitmap dimensions 
//
// POST CONDITIONS:
// Returns TRUE if successful
// 
// ERROR CONDITIONS:
// All errors exit thorugh DDInitFailed() function
//***************************************************************
BOOL DDShowBitmap( LPDIRECTDRAWSURFACE4 lpDDS, HBITMAP hBitmap,
				   int xLocation, int yLocation,
				   int bWidth, int bHeight)
{

    HDC             hdcImage = NULL;
    HDC             hdcSurf  = NULL;
	HDC             thisDevice = NULL;

    // Create a DC and select the image into it.
    hdcImage = CreateCompatibleDC( NULL );
    SelectObject( hdcImage, hBitmap);
   
    // Get a DC for the surface.
	if(lpDDS->GetDC(&hdcSurf) != DD_OK)
		DDInitFailed(hWnd, hRet,
		            "GetDC() call failed in DDShowBitmap()");
  
    // BitBlt() is used to display bitmap
    if ( BitBlt( hdcSurf, xLocation, yLocation, bWidth,
		         bHeight, hdcImage, 0, 0, SRCCOPY )
				 == FALSE ) 
		DDInitFailed(hWnd, hRet,
		            "BitBlt() call failed in DDShowBitmap()");

    // Release device contexts
    if ( hdcSurf )
        lpDDS->ReleaseDC( hdcSurf );
    if ( hdcImage )
        DeleteDC( hdcImage );

	return TRUE;
}

//***************************************************************
// Name: DDBmapToSurf
// Desc: Moves a bitmap to a DirectDraw Surface
// PRE CONDITIONS:
// 1. Parameter 1 is pointer to a DirectDraw surface
//    Parameter 2 is handle to the bitmap
//
// POST CONDITIONS:
// Bitmap is move to surface
// Returns 1 if successful
// /
// ERROR CONDITIONS:
// All errors exit thorugh DDInitFailed() function
//***************************************************************
HRESULT DDBmapToSurf(LPDIRECTDRAWSURFACE4 pdds, HBITMAP hbm)
{
    HDC                     hdcImage;
    HDC                     hdc;
    DDSURFACEDESC2          ddsd;
    HRESULT                 hr = 1;
	BOOL					retValue;

    if (hbm == NULL || pdds == NULL)
 		DDInitFailed(hWnd, hRet,
		       "Invalid surface or bitmap in DDBmapToSurf");

   // Create compatible DC and select bitmap into it
	hdcImage = CreateCompatibleDC(NULL);
    if (!hdcImage)
 		DDInitFailed(hWnd, hRet,
		       "CreateCompatibleDC() failed in DDBmapToSurf");
    SelectObject(hdcImage, hbm);
 
    // Get size of surface
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    pdds->GetSurfaceDesc(&ddsd);

    if ((hr = pdds->GetDC(&hdc)) != DD_OK)
 		DDInitFailed(hWnd, hRet,
		       "GetDC() failed in DDBmapToSurf");

	 retValue = BitBlt(hdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight,
		               hdcImage, 0, 0, SRCCOPY);
	 // Release surface immediately
	 pdds->ReleaseDC(hdc);

	 if(retValue == FALSE)
   		DDInitFailed(hWnd, hRet,
		       "BitBlt() failed in DDBmapToSurf");

        DeleteDC(hdcImage);
    return hr;
}


