/**********************************************************************
  PROJECT: DD Multi Sprite Animation
  Source: DD Multi Sprite Animation.cpp
  Chapter reference: 16
  Interface: DirectX 7 or later
  Date: June 12/2001
***********************************************************************
  Description: 
   DirectDraw animation by flipping
   Topics:
    1. Creating a primary surface with a back buffer
	2. Attaching a clipper to primary surface and back buffer
	3. Storing and accessing individualized sprite data 
    4. Animating sprites by flipping 
	5. Obtaining and using dynamic color keys
***********************************************************************/

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN  // Exclude unnecessary window.h components
#endif

#include <windows.h>
#include <ddraw.h>
#include <stdio.h>
#include <stdarg.h>


//Global variables
LPDIRECTDRAW7				lpDD7 = NULL;        // DirectDraw object
LPDIRECTDRAWSURFACE7        lpDDSPrimary = NULL; // DirectDraw primary 
LPDIRECTDRAWSURFACE7        lpDDSBackBuf = NULL; // Back buffer pointer
LPDIRECTDRAWSURFACE7		lpDDSBmap1 = NULL;   // Balloon1 surface
LPDIRECTDRAWSURFACE7		lpDDSBmap2 = NULL;   // Balloon2 surface
LPDIRECTDRAWSURFACE7		lpDDSBmap3 = NULL;   // Balloon3 surface
LPDIRECTDRAWSURFACE7        lpDDSBackGrnd = NULL; // Background surface
LPDIRECTDRAWCLIPPER			lpDDClipper;         // Clipper object
DDSURFACEDESC2              ddsd;                // Surface description
DDSCAPS2                    ddscaps;             // Surface caps
HWND                        hWnd;
HRESULT                     hRet;


// Video display globals
LONG		vidPitch = 0;	// Pitch
LPVOID		vidStart = 0;	// Buffer address


// DirectDraw modes data
int modesCount = 0;        // Counter for DD modes
static int MAX_MODES = 50; // Maximum number of modes
static int MODES_ON = 0;   // Display modes are stored
DWORD modesArray[150];     // Array for mode paramenters

// Bitmap data
HBITMAP		bal1Bitmap;    // Handle to balloon1
HBITMAP		bal2Bitmap;    // Handle to balloon2
HBITMAP		bal3Bitmap;    // Handle to balloon2
HBITMAP     bgBitmap;      // Handle to background
BITMAP		bMap1;         // Data for balloon1
BITMAP      bMap2;         // Balloon 2
BITMAP      bMap3;         // Balloon 2
BITMAP      bMapbg;        // Data for background

// Variables for available memory
DDSCAPS2		ddCapsMem;
DWORD           memTotal;
DWORD           memFree;
DWORD			MEM_REQUIRED = 2200000;

// Color key data
DDCOLORKEY	bColorKey;
WORD		dynamicKey;

// Timer control variables
static int		   apOn;
_int64      	   thisTickCount;
_int64             lastTickCount;
_int64             frequency;
_int64             TIMER_VALUE = 15; // milliseconds


// Sprite control structure and variables
struct SpriteCtrl
{
   LONG		startY;          // Start x coodintate
   LONG		startX;          // Start y coordinate
   LONG     bmapX;           // Width of bitmap
   LONG     bmapY;           // Height of bitmap
   LONG     iterMax;         // Maximum iteration count
   LONG		skipFactor;      // Display delay
   LONG     iteration;       // Sprite iteration counter
} Sprite1, Sprite2, Sprite3;

   
//*************************************
//            Prototypes
//*************************************
void ReleaseObjects(void);
HRESULT DDInitFailed(HWND, HRESULT, LPCTSTR);
void UpdateFrame(HWND);
long CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
int DD7Interface();
int HasDDMode(DWORD, DWORD, DWORD);
HRESULT WINAPI ModesProc(LPDDSURFACEDESC2, LPVOID);
int LockSurface(LPDIRECTDRAWSURFACE7);
void ReleaseSurface(LPDIRECTDRAWSURFACE7);
void FlipImages();
void SpriteAction(SpriteCtrl &, LPDIRECTDRAWSURFACE7);
HRESULT DDBmapToSurf(LPDIRECTDRAWSURFACE7, HBITMAP);


//*****************************************************************
// WinMain implementation for exclusive mode DirectDraw application
//*****************************************************************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
    MSG                         msg;

    static char szAppName[] = "DDAppName" ; // Application name



     // Defining a structure of type WNDCLASSEX
     WNDCLASSEX  wndclass ;
     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject
                              (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName;
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;
	 
	 // Register the class
	 RegisterClassEx(&wndclass);

    // Create a window
    hWnd = CreateWindowEx(WS_EX_TOPMOST,  // Extended style
                szAppName,
                "DirectDraw Demo No. 2",
                WS_POPUP,       // Window style
                0,              // Horizontal origin  
                0,              // Vertical origin
                GetSystemMetrics(SM_CXSCREEN), // x size
                GetSystemMetrics(SM_CYSCREEN), // y size
                NULL,            // Handle of parent
                NULL,            // Handle to menu
                hInstance,       // Application instance
                NULL);           // Additional data

    if (!hWnd)
        return FALSE;

    ShowWindow(hWnd, nCmdShow);

    //***************************************
    // Create DirectDraw object and set mode
    //***************************************
    // Fetch DirectDraw7 interface
	hRet = DD7Interface();
	if (hRet == 0)
		return DDInitFailed(hWnd, hRet,
		                    "QueryInterface() call failed");

    // Check for necessary free video memory
	    ZeroMemory(&ddCapsMem, sizeof(ddCapsMem)); 
    	lpDD7->GetAvailableVidMem(&ddCapsMem, &memTotal, &memFree);
        if (memTotal <	MEM_REQUIRED)
            return DDInitFailed(hWnd, hRet,
		         "Insufficient video memory");

	// Check for available mode (640 by 480 in 16-bit color)
	if (HasDDMode(640, 480, 16) == 0) 
        return DDInitFailed(hWnd, hRet, "Display mode not available");
	
    // Set cooperative level to exclusive and full screen
    hRet = lpDD7->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE
		                              | DDSCL_FULLSCREEN);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetCooperativeLevel() call failed");
    // Set the video mode to 640 x 480 x 16
    hRet = lpDD7->SetDisplayMode(640, 480, 16, 0, 0);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetDisplayMode() call failed");



    //******************************************************
    // Create primary surface and one or more back buffers
    //******************************************************
	// ddsd is a structure of type DDSRUFACEDESC2
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE |
		                  DDSCAPS_FLIP |
						  DDSCAPS_COMPLEX |
						  DDSCAPS_VIDEOMEMORY;
	// Request two back buffers
	ddsd.dwBackBufferCount = 2; 
    hRet = lpDD7->CreateSurface(&ddsd, &lpDDSPrimary, NULL);
	// If call failed, retry with one back buffer
		if (hRet == DDERR_OUTOFVIDEOMEMORY)	{
			ddsd.dwBackBufferCount = 1; 
			hRet = lpDD7->CreateSurface(&ddsd, &lpDDSPrimary, NULL);
		}
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "CreateSurface() call failed");
	// Get backbuffer pointer
    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
	hRet = lpDDSPrimary->GetAttachedSurface(&ddscaps,
		                 &lpDDSBackBuf);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "Backbuffer pointer not obtained");


    //***************************************************
	//  Create a clipper and attach to primary surface
	//  and to the back buffer
	//***************************************************
    hRet = lpDD7->CreateClipper(0, &lpDDClipper, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "Create clipper failed");
	// Associate clipper with application window
	hRet = lpDDClipper->SetHWnd(0, hWnd);
    if (hRet != DD_OK)
		return DDInitFailed(hWnd, hRet,
		                    "Clipper not linked to application window");
    // Associate clipper with primary surface
	hRet = lpDDSPrimary->SetClipper(lpDDClipper);
    if (hRet != DD_OK)
		return DDInitFailed(hWnd, hRet,
		                    "Clipper not linked to primary surface");

    // Associate clipper with back buffer surface
	hRet = lpDDSBackBuf->SetClipper(lpDDClipper);
    if (hRet != DD_OK)
		return DDInitFailed(hWnd, hRet,
		                    "Clipper not linked to back buffer surface");

	//*************************************************
	// move first balloon bitmap to off-screen surface
	//*************************************************
	// Load the bitmap into memory
	bal1Bitmap = ( HBITMAP )LoadImage( NULL, "balloon1.bmp", 
            IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE );
    if ( bal1Bitmap == NULL ) 
         DDInitFailed(hWnd, hRet,
		      "Balloon1 bitmap load failed");
	// Get bitmap dimensions to determine surface size
	GetObject(bal1Bitmap, sizeof (BITMAP), &bMap1);

    // Create the off-screen surface for bitmap in system memory
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
    ddsd.dwHeight = bMap1.bmHeight;
	ddsd.dwWidth = bMap1.bmWidth;
    hRet = lpDD7->CreateSurface(&ddsd, &lpDDSBmap1, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "Off screen surface1 creation failed ");
	// Move bitmap to surface using DDBmapToSurf()function 
	hRet = DDBmapToSurf(lpDDSBmap1, bal1Bitmap);
    if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "DDMapToSurf() call failed");

	//*****************************************
	//     read color key from loaded sprite
    //*****************************************
	// Attempt to lock the surface for direct access
	if (!LockSurface(lpDDSBmap1))
        return DDInitFailed(hWnd, hRet,
		                    "Surface Lock failed ");

	// 	  LONG			vidPitch;	// Pitch
	//	  LPVOID		vidStart;	// Buffer address
	_asm
	{
		    PUSH	ESI				  ; Save context
			PUSHF	
			MOV		ESI,vidStart
	; Read and store pixel attributes
			MOV		AX,[ESI]		; Get attribute
			MOV		dynamicKey,AX   ; store in variable
			POPF					; Restore context
			POP		ESI
	}

	ReleaseSurface(lpDDSBmap1);

	// Set color key for balloon1 surface
	 bColorKey.dwColorSpaceLowValue = dynamicKey;
	 bColorKey.dwColorSpaceHighValue = dynamicKey;
     hRet = lpDDSBmap1->SetColorKey(DDCKEY_SRCBLT, &bColorKey);
     if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetColorKey() for Balloon1 failed");
  

	//*************************************************
	// move second balloon bitmap to off-screen surface
	//*************************************************
	// Load the itmap into memory
	bal2Bitmap = ( HBITMAP )LoadImage( NULL, "balloon2.bmp", 
            IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE );
    if ( bal2Bitmap == NULL ) 
         DDInitFailed(hWnd, hRet,
		      "Balloon1 bitmap load failed");
	// Get bitmap dimensions to determine surface size
	GetObject(bal2Bitmap, sizeof (BITMAP), &bMap2);

    // Create the off-screen surface for bitmap in system memory
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
    ddsd.dwHeight = bMap2.bmHeight;
	ddsd.dwWidth = bMap2.bmWidth;
    hRet = lpDD7->CreateSurface(&ddsd, &lpDDSBmap2, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "Off screen surface1 creation failed ");

	// Set color key for balloon2 surface using the same
	// dynamic color key obtained for balloon1
	 bColorKey.dwColorSpaceLowValue = dynamicKey;
	 bColorKey.dwColorSpaceHighValue = dynamicKey;
     
	 hRet = lpDDSBmap2->SetColorKey(DDCKEY_SRCBLT, &bColorKey);
     if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetColorKey() for Balloon1 failed");
    
     // Move bitmap to surface using DDBmapToSurf()function 
	 hRet = DDBmapToSurf(lpDDSBmap2, bal2Bitmap);
     if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "DDMapToSurf() call failed");	 
	 
	//*************************************************
	// move third balloon bitmap to off-screen surface
	//*************************************************
	// Load the itmap into memory
	bal3Bitmap = ( HBITMAP )LoadImage( NULL, "balloon3.bmp", 
            IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE );
    if ( bal3Bitmap == NULL ) 
         DDInitFailed(hWnd, hRet,
		      "Balloon3 bitmap load failed");
	// Get bitmap dimensions to determine surface size
	GetObject(bal3Bitmap, sizeof (BITMAP), &bMap3);

    // Create the off-screen surface for bitmap in system memory
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
    ddsd.dwHeight = bMap3.bmHeight;
	ddsd.dwWidth = bMap3.bmWidth;
    hRet = lpDD7->CreateSurface(&ddsd, &lpDDSBmap3, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "Off screen surface creation failed ");

    // Set color key for balloon3 surface using the same
	// dynamic color key obtained for balloon1
	 bColorKey.dwColorSpaceLowValue = dynamicKey;
	 bColorKey.dwColorSpaceHighValue = dynamicKey;
     hRet = lpDDSBmap3->SetColorKey(DDCKEY_SRCBLT, &bColorKey);
     if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetColorKey() for Balloon 3 failed");
    
     // Move bitmap to surface using DDBmapToSurf()function 
	 hRet = DDBmapToSurf(lpDDSBmap3, bal3Bitmap);
     if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "DDMapToSurf() call failed");	
	 

	//**********************************************
	//  move background bitmap to offscreen surface
	//**********************************************
 	// Load the bitmap into memory
	bgBitmap = ( HBITMAP )LoadImage( NULL, "backgrnd.bmp", 
            IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE );
    if ( bgBitmap == NULL ) 
         DDInitFailed(hWnd, hRet,
		      "Background bitmap load failed");
	// Get bitmap dimensions to determine surface size
	GetObject(bgBitmap, sizeof (BITMAP), &bMapbg);

    // Create the off-screen surface for bitmap in system memory
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
    ddsd.dwHeight = bMapbg.bmHeight;
	ddsd.dwWidth = bMapbg.bmWidth;
    hRet = lpDD7->CreateSurface(&ddsd, &lpDDSBackGrnd, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		       "Off screen surface (background) creation failed ");
  
     // Move bitmap to surface using DDBmapToSurf()function 
	 hRet = DDBmapToSurf(lpDDSBackGrnd, bgBitmap);
     if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "DDMapToSurf() call failed");

	 //******************************************************
	 //   Fill SpriteCtrl structure members for each sprite
     //******************************************************
	 // Sprite1 is balloon bitmap in bMap1
	 // Resolution is 640 by 480 pixles
	 Sprite1.startY = 479;     // Starts at screen bottom
	 Sprite1.startX = 70;      // x for start position
	 Sprite1.bmapX = bMap1.bmWidth;
	 Sprite1.bmapY = bMap1.bmHeight;
	 Sprite1.skipFactor = 2;
	 Sprite1.iterMax = (480+(bMap1.bmHeight)) * Sprite1.skipFactor;
	 Sprite1.iteration = 50;    // Init iteration counter


	 // Sprite2 is balloon bitmap in bMap2
	 Sprite2.startY = 479;     // Starts at screen bottom
	 Sprite2.startX = 240;     // x for start position
	 Sprite2.bmapX = bMap2.bmWidth;
	 Sprite2.bmapY = bMap2.bmHeight;
	 Sprite2.skipFactor = 1;
	 Sprite2.iterMax = 480+(bMap2.bmHeight);
	 Sprite2.iteration = 50;    // Init iteration counter

	 // Sprite3 is balloon bitmap in bMap3
	 Sprite3.startY = 479;     // Starts at screen bottom
	 Sprite3.startX = 500;      // x for start position
	 Sprite3.bmapX = bMap3.bmWidth;
	 Sprite3.bmapY = bMap3.bmHeight;
	 Sprite3.skipFactor = 3;
	 Sprite3.iterMax = (480+(bMap3.bmHeight)) * Sprite3.skipFactor;
	 Sprite3.iteration = 20;    // Init iteration counter	 

     // Establish timer frequency and adjust counter
	 QueryPerformanceFrequency((LARGE_INTEGER*) &frequency);
     TIMER_VALUE = (TIMER_VALUE * frequency) / 1000;
     
	 // Start bitmap display
	 apOn = 1;

    while ( 1 )
    {
        if ( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if ( !GetMessage( &msg, NULL, 0, 0 ) )
            {
                return msg.wParam;
            }
            TranslateMessage( &msg );
            DispatchMessage( &msg );
        }
        else if ( apOn == 1 )  
           FlipImages();
        
        else
           WaitMessage();
     }

}

//*****************************************************************
// Name: WndProc
// Desc: Window procedure
//*****************************************************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam,
LPARAM lParam)
{

    switch (message)
    {
        case WM_DESTROY:
            // Clean up and close the app
            ReleaseObjects();
            PostQuitMessage(0);
            return 0L;
			break;

        case WM_KEYDOWN:
            // Handle keystroke to end execution
            switch (wParam)
            {
                case VK_ESCAPE:
                    PostMessage(hwnd, WM_CLOSE, 0, 0);
                    return 0L;
					break;
			
			}

        case WM_SETCURSOR:
            // Turn off the cursor since this is a full-screen app
            SetCursor(NULL);
            return TRUE;
        
	  }
    return DefWindowProc(hwnd, message, wParam, lParam);
}


//************************************
// Name: ReleaseObject
// Desc: Releases all objects
//************************************
static void ReleaseObjects(void)
{
    if (lpDD7 != NULL)
    {
        if (lpDDSPrimary != NULL)
        {
            lpDDSPrimary->Release();
            lpDDSPrimary = NULL;
			lpDDSBmap1->Release();
			lpDDSBmap2->Release();
			lpDDSBmap3->Release();
			lpDDSBackGrnd->Release();
			lpDDClipper->Release();
        }
        lpDD7->Release();
        lpDD7 = NULL;
    }
}


//*************************************************
// Name: FlipImages
// Desc: Update back buffer and flip
//*************************************************

static void FlipImages()
{

   QueryPerformanceCounter((LARGE_INTEGER*) &thisTickCount);
   if((thisTickCount - lastTickCount) < TIMER_VALUE)
	   return;

   else
   {
   	  lastTickCount = thisTickCount;

      // Blit background bitmap to back buffer
	  hRet = lpDDSBackBuf->Blt(NULL,
		                      lpDDSBackGrnd,
							  NULL,
							  DDBLT_WAIT,
							  NULL);
     if(hRet != DD_OK){
        DDInitFailed(hWnd, hRet,
	                "Blt() on background failed");
		    return;
		}

      // Animate sprites. Farthest ones first  
      SpriteAction(Sprite3, lpDDSBmap3);
	  SpriteAction(Sprite2, lpDDSBmap2);
	  SpriteAction(Sprite1, lpDDSBmap1);

	 // Flip surfaces
     hRet = lpDDSPrimary->Flip(NULL, DDFLIP_WAIT);
     if(hRet != DD_OK){
        DDInitFailed(hWnd, hRet,
	                "Flip() call failed");
		      return;
		}
		return;
   }
}

//************************************************
// Name: SpriteAction
// Desc: Animate a sprite according
//       to its own parameters
// PRE CONDITIONS:
//    1. Pointer to structure containing
//       sprite data
//    2. Pointer to DirectDraw suface
//       containing sprite bitmap
//***********************************************
void SpriteAction(SpriteCtrl &thisSprite,
				  LPDIRECTDRAWSURFACE7 lpDDSBmap)
{

    RECT				destSurf;
    LONG                vertUpdate;

	thisSprite.iteration++;

	if(thisSprite.iteration == thisSprite.iterMax)
		thisSprite.iteration = 0;

	vertUpdate = thisSprite.iteration / thisSprite.skipFactor;

     // Set coordinates for balloon1 display
     destSurf.left = thisSprite.startX;
	 destSurf.top = thisSprite.startY - vertUpdate;
	 destSurf.right = destSurf.left + thisSprite.bmapX;
	 destSurf.bottom = destSurf.top + thisSprite.bmapY;

	 // Use Blt() to blit bitmap from the off-screen surface
	 // (->lpDDSBitamp), onto the back buffer (->lpDDSBackBuf)
	 hRet = lpDDSBackBuf->Blt(&destSurf,
		                      lpDDSBmap,
							  NULL,
							  DDBLT_WAIT | DDBLT_KEYSRC,
							  NULL);

     if(hRet != DD_OK){
        DDInitFailed(hWnd, hRet,
	                "Blt() on sprite failed");
		return;
	 }

   return;

}

//***************************************
// Name: DDInitFailed()
// Desc: This function is called if an
//       initialization call fails
//***************************************
HRESULT DDInitFailed(HWND hWnd, HRESULT hRet, LPCTSTR szError)
{
    char   szBuff[128];

    sprintf(szBuff, szError);
    ReleaseObjects();
    MessageBox(hWnd, szBuff, "DD Multi Sprite Animation", MB_OK);
    DestroyWindow(hWnd);
    return hRet;

}

//******************************************
// Name: DD7Interface()
// Desc: Finds DirectDraw7 object
// 
// POST CONDITIONS:
//   returns 0 if no DirectDraw7 interface
//   returns 1 if DirectDraw7 found
//******************************************
int DD7Interface()
{

	HRESULT         DDConnect;
	LPDIRECTDRAW	lpDD;    // Pointer to DirectDraw

	DDConnect = DirectDrawCreate ( NULL,
	           &lpDD,
			   NULL);
     
	if(DDConnect != DD_OK)
		return 0;
    // Atempt to locate DirectDraw4 interface
    DDConnect = lpDD->QueryInterface(
                IID_IDirectDraw7,
	            (LPVOID *) &lpDD7);
    if(DDConnect != S_OK)
		return 0;

    lpDD->Release();      // Release old pointer
        return 1;

}


//***********************************************
// Name: ModesProc
// Desc: Callback function for EnumDisplayModes()
//***********************************************
HRESULT WINAPI ModesProc(LPDDSURFACEDESC2 aSurface,
								  LPVOID Context)
{
   static int i;         // Index into array of mode data

   i = modesCount * 3;   // Set array pointer

   if( modesCount >= MAX_MODES )
	   return DDENUMRET_CANCEL;  // Stop mode listing

   // Store mode paramenters in public array
   // Note: code assumes that the dwRGBBitFormat member if
   //       the DDPIXELFORMAT structure contains valid data
   modesArray[i]     = aSurface->dwWidth;
   modesArray[i + 1] = aSurface->dwHeight;
   modesArray[i + 2] = aSurface->ddpfPixelFormat.dwRGBBitCount;

   modesCount++;         // Bump display modes counter
   return DDENUMRET_OK;  // Continue

}


//***************************************************************
// Name: hasDDMode
// Desc: Tests for mode availability
//***************************************************************
// PRE CONDITIONS:
// 1. Public array modesArray[] to store mode data
// 2. Public int variable modesCount to store number of
//    display modes
// 3. ModesProc() is an associated function that stores
//    mode data in array and count modes
//
// POST CONDITIONS:
//  1. Returns 1 if mode is available
//***************************************************************

int HasDDMode(DWORD pixWidth, DWORD pixHeight, DWORD pixBits)
{

	static HRESULT DDConnect;

    // Call EnumDisplayModes() 
	if(MODES_ON == 0){
		MODES_ON = 1;         // set switch
		DDConnect = lpDD7->EnumDisplayModes(0, NULL, NULL,
			                                ModesProc);
	}
// Modes are now stored in modeArray[] as triplets encoding
// width, height, and pixel bit size
// Variable modesCount holds the total number of display modes
	for(int x = 0; x < (modesCount * 3); x += 3){
		if(modesArray[x]==pixWidth && modesArray[x+1]==pixHeight\
			&& modesArray[x+2]==pixBits)
			return 1;
	}
	return 0;
}


//***************************************************************
// Name: LockSurface
// Desc: Function to lock a DirectDraw surface and store the
//       direct access parameters
// PRE CONDITIONS:
// 1. Parameter is pointer to DirectDraw surface
// 2. Video display globals have been declared as follows:
// 	  LONG			vidPitch;	// Pitch
//	  LPVOID		vidStart;	// Buffer address
//
// POST CONDITIONS:
// Returns 1 if call succeeds and 0 if it failed
//***************************************************************
int LockSurface(LPDIRECTDRAWSURFACE7  lpSurface) 
{
	// Attempt to lock the surface for direct access
	if (lpSurface->Lock(NULL, &ddsd, DDLOCK_WAIT, NULL)\
		 != DD_OK)
		return 0;                  // Lock failed
	// Store video system data
    vidPitch = ddsd.lPitch;        // Pitch
	vidStart = ddsd.lpSurface;     // Buffer address	
    return 1;					   // Surface locked   
}

//***************************************************************
// Name: ReleaseSurface
// Desc: Function to release locked surface 
// PRE CONDITIONS:
// 1. Parameter is pointer to DirectDraw surface
// 2. Video display globals as follows:
// 	  LONG			vidPitch;	// Pitch
//	  LPVOID		vidStart;	// Buffer address
//***************************************************************
void ReleaseSurface(LPDIRECTDRAWSURFACE7  lpSurface) 
{
	lpSurface->Unlock(NULL);
	// Clear global variables
	vidPitch = 0;
	vidStart = 0;
    return;
}



//***************************************************************
// Name: DDBmapToSurf
// Desc: Moves a bitmap to a DirectDraw Surface
// PRE CONDITIONS:
// 1. Parameter 1 is pointer to a DirectDraw surface
//    Parameter 2 is handle to the bitmap
//
// POST CONDITIONS:
// Bitmap is move to surface
// Returns 1 if successful
// /
// ERROR CONDITIONS:
// All errors exit thorugh DDInitFailed() function
//***************************************************************
HRESULT DDBmapToSurf(LPDIRECTDRAWSURFACE7 pdds, HBITMAP hbm)
{
    HDC                     hdcImage;
    HDC                     hdc;
    DDSURFACEDESC2          ddsd;
    HRESULT                 hr = 1;
	BOOL					retValue;

    if (hbm == NULL || pdds == NULL)
 		DDInitFailed(hWnd, hRet,
		       "Invalid surface or bitmap in DDBmapToSurf");

   // Create compatible DC and select bitmap into it
	hdcImage = CreateCompatibleDC(NULL);
    if (!hdcImage)
 		DDInitFailed(hWnd, hRet,
		       "CreateCompatibleDC() failed in DDBmapToSurf");
    SelectObject(hdcImage, hbm);
 
    // Get size of surface
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    pdds->GetSurfaceDesc(&ddsd);

    if ((hr = pdds->GetDC(&hdc)) != DD_OK)
 		DDInitFailed(hWnd, hRet,
		       "GetDC() failed in DDBmapToSurf");

	 retValue = BitBlt(hdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight,
		               hdcImage, 0, 0, SRCCOPY);
	 // Release surface immediately
	 pdds->ReleaseDC(hdc);

	 if(retValue == FALSE)
   		DDInitFailed(hWnd, hRet,
		       "BitBlt() failed in DDBmapToSurf");

        DeleteDC(hdcImage);
    return hr;
}


