/**********************************************************************
  PROJECT: DD Panning Demo
  Source: DD_Pan.cpp
  Chapter reference: 12
***********************************************************************
  Description: 
   DirectDraw panning animation

   Topics:
    1. Creating a primary surface with a back buffer
	2. Flipping primary surface and backbuffer
***********************************************************************/

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN  // Exclude unnecessary window.h components
#endif

#include <windows.h>
#include <ddraw.h>
#include <stdio.h>
#include <stdarg.h>


//Global variables
LPDIRECTDRAW4				lpDD4 = NULL;        // DirectDraw object
LPDIRECTDRAWSURFACE4        lpDDSPrimary = NULL; // DirectDraw primary 
LPDIRECTDRAWSURFACE4        lpDDSBackBuf = NULL; // Back buffer pointer
LPDIRECTDRAWSURFACE4		lpDDSBmap1 = NULL;   // Balloon1 surface
LPDIRECTDRAWSURFACE4		lpDDSBmap2 = NULL;   // Balloon2 surface
LPDIRECTDRAWSURFACE4		lpDDSBmap3 = NULL;   // Balloon3 surface
LPDIRECTDRAWSURFACE4        lpDDSBackGrnd = NULL; // Background surface
LPDIRECTDRAWCLIPPER			lpDDClipper;         // Clipper object
DDSURFACEDESC2              ddsd;                // Surface description
DDSCAPS2                    ddscaps;             // Surface caps
HWND                        hWnd;
HRESULT                     hRet;

// Video display globals
LONG		vidPitch = 0;	// Pitch
LPVOID		vidStart = 0;	// Buffer address


// DirectDraw modes data
int modesCount = 0;        // Counter for DD modes
static int MAX_MODES = 50; // Maximum number of modes
static int MODES_ON = 0;   // Display modes are stored
DWORD modesArray[150];     // Array for mode paramenters

// Bitmap data
HBITMAP     bgBitmap;      // Handle to background
BITMAP      bMapbg;        // Data for background


// Timer control variables
DWORD			   thisTickCount;
DWORD			   lastTickCount;
static DWORD       TIMER_VALUE = 15;
static int		   apOn;

// Panning animation controls 
RECT            thisPan;   // Storage for source rectangle
LONG			panIteration = 0;  // panning iteration counter
LONG			panDirection = 0;  // 1 = left, 0 = right
// Constants
LONG			PAN_LIMIT_LEFT = 1;
LONG			PAN_LIMIT_RIGHT = 639;

//*************************************
//            Prototypes
//*************************************
void ReleaseObjects(void);
HRESULT DDInitFailed(HWND, HRESULT, LPCTSTR);
void UpdateFrame(HWND);
long CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
int DD4Interface();
int HasDDMode(DWORD, DWORD, DWORD);
HRESULT WINAPI ModesProc(LPDDSURFACEDESC2, LPVOID);
int LockSurface(LPDIRECTDRAWSURFACE4);
void ReleaseSurface(LPDIRECTDRAWSURFACE4);
void PanImage();
HRESULT DDBmapToSurf(LPDIRECTDRAWSURFACE4, HBITMAP);


//*****************************************************************
// WinMain implementation for exclusive mode DirectDraw application
//*****************************************************************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
    MSG                         msg;

    static char szAppName[] = "DDAppName" ; // Application name



     // Defining a structure of type WNDCLASSEX
     WNDCLASSEX  wndclass ;
     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject
                              (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName;
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;
	 
	 // Register the class
	 RegisterClassEx(&wndclass);

    // Create a window
    hWnd = CreateWindowEx(WS_EX_TOPMOST,  // Extended style
                szAppName,
                "DirectDraw Demo No. 2",
                WS_POPUP,       // Window style
                0,              // Horizontal origin  
                0,              // Vertical origin
                GetSystemMetrics(SM_CXSCREEN), // x size
                GetSystemMetrics(SM_CYSCREEN), // y size
                NULL,            // Handle of parent
                NULL,            // Handle to menu
                hInstance,       // Application instance
                NULL);           // Additional data

    if (!hWnd)
        return FALSE;

    ShowWindow(hWnd, nCmdShow);

    //***************************************
    // Create DirectDraw object and set mode
    //***************************************
    // Fetch DirectDraw4 interface
	hRet = DD4Interface();
	if (hRet == 0)
		return DDInitFailed(hWnd, hRet,
		                    "QueryInterface() call failed");

	// Check for available mode (640 by 480 in 16-bit color)
	if (HasDDMode(640, 480, 16) == 0) 
        return DDInitFailed(hWnd, hRet, "Display mode not available");
	
    // Set cooperative level to exclusive and full screen
    hRet = lpDD4->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE
		                              | DDSCL_FULLSCREEN);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetCooperativeLevel() call failed");
    // Set the video mode to 640 x 480 x 16
    hRet = lpDD4->SetDisplayMode(640, 480, 16, 0, 0);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetDisplayMode() call failed");

    //*********************************************
    // Create primary surface and two back buffers
    //*********************************************
	// ddsd is a structure of type DDSRUFACEDESC2
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE |
		                  DDSCAPS_FLIP |
						  DDSCAPS_COMPLEX |
						  DDSCAPS_VIDEOMEMORY;
	// Request two back buffers
	ddsd.dwBackBufferCount = 1; 
    hRet = lpDD4->CreateSurface(&ddsd, &lpDDSPrimary, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "CreateSurface() call failed");
	// Get backbuffer pointer
    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
	hRet = lpDDSPrimary->GetAttachedSurface(&ddscaps,
		                 &lpDDSBackBuf);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "Backbuffer pointer not obtained");


    //***************************************************
	//  Create a clipper and attach to primary surface
	//  and to the back buffer
	//***************************************************
    hRet = lpDD4->CreateClipper(0, &lpDDClipper, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "Create clipper failed");
	// Associate clipper with application window
	hRet = lpDDClipper->SetHWnd(0, hWnd);
    if (hRet != DD_OK)
		return DDInitFailed(hWnd, hRet,
		                    "Clipper not linked to application window");
    // Associate clipper with primary surface
	hRet = lpDDSPrimary->SetClipper(lpDDClipper);
    if (hRet != DD_OK)
		return DDInitFailed(hWnd, hRet,
		                    "Clipper not linked to primary surface");

    // Associate clipper with back buffer surface
	hRet = lpDDSBackBuf->SetClipper(lpDDClipper);
    if (hRet != DD_OK)
		return DDInitFailed(hWnd, hRet,
		                    "Clipper not linked to back buffer surface");


	//**********************************************
	//  move background bitmap to offscreen surface
	//**********************************************
 	// Load the itmap into memory
	bgBitmap = ( HBITMAP )LoadImage( NULL, "panback.bmp", 
            IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE );
    if ( bgBitmap == NULL ) 
         DDInitFailed(hWnd, hRet,
		      "Background bitmap load failed");
	// Get bitmap dimensions to determine surface size
	GetObject(bgBitmap, sizeof (BITMAP), &bMapbg);

    // Create the off-screen surface for bitmap in system memory
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
    ddsd.dwHeight = bMapbg.bmHeight;
	ddsd.dwWidth = bMapbg.bmWidth;
    hRet = lpDD4->CreateSurface(&ddsd, &lpDDSBackGrnd, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		       "Off screen surface (background) creation failed ");

    
     // Move bitmap to surface using DDBmapToSurf()function 
	 hRet = DDBmapToSurf(lpDDSBackGrnd, bgBitmap);
     if(hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "DDMapToSurf() call failed");

	 
     // Start bitmap display
	 apOn = 1;

    while ( 1 )
    {
        if ( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if ( !GetMessage( &msg, NULL, 0, 0 ) )
            {
                return msg.wParam;
            }
            TranslateMessage( &msg );
            DispatchMessage( &msg );
        }
        else if ( apOn == 1 )  
           PanImage();
        
        else
           WaitMessage();
     }

}

//*****************************************************************
// Name: WndProc
// Desc: Window procedure
//*****************************************************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam,
LPARAM lParam)
{

    switch (message)
    {
        case WM_DESTROY:
            // Clean up and close the app
            ReleaseObjects();
            PostQuitMessage(0);
            return 0L;
			break;

        case WM_KEYDOWN:
            // Handle keystroke to end execution
            switch (wParam)
            {
                case VK_ESCAPE:
                    PostMessage(hwnd, WM_CLOSE, 0, 0);
                    return 0L;
					break;
			
			}

        case WM_SETCURSOR:
            // Turn off the cursor since this is a full-screen app
            SetCursor(NULL);
            return TRUE;
        
	  }
    return DefWindowProc(hwnd, message, wParam, lParam);
}


//************************************
// Name: ReleaseObject
// Desc: Releases all objects
//************************************
static void ReleaseObjects(void)
{
    if (lpDD4 != NULL)
    {
        if (lpDDSPrimary != NULL)
        {
            lpDDSPrimary->Release();
            lpDDSPrimary = NULL;
			lpDDSBackGrnd->Release();
			lpDDClipper->Release();
        }
        lpDD4->Release();
        lpDD4 = NULL;
    }
}


//*************************************************
// Name: PanImage
// Desc: Update back with a source rectagle that
//       is a portion of the background bitmap
//       and flip to create a panning animation
//*************************************************

static void PanImage()
{

   thisTickCount = GetTickCount();
   if((thisTickCount - lastTickCount) < TIMER_VALUE)
	   return;

   else
   {
   	  lastTickCount = thisTickCount;

	  // Bump pan iteration according to direction
	  if(panDirection == 1)
		  panIteration--;
	  else
		  panIteration++;
	  
	  // Reset panning iteration counter at limits
	  if(panIteration == PAN_LIMIT_RIGHT)
		  panDirection = 1;          // Pan left
	  if(panIteration == PAN_LIMIT_LEFT)
		  panDirection = 0;
	  
	  // Set panning rectangle in source image
	  thisPan.left = panIteration;
	  thisPan.top = 0;
	  thisPan .right = 640 + panIteration;
	  thisPan.bottom = 480;

      // Blit background bitmap to back buffer
	  hRet = lpDDSBackBuf->Blt(NULL,
		                      lpDDSBackGrnd,
							  &thisPan,
							  DDBLT_WAIT,
							  NULL);
     if(hRet != DD_OK){
        DDInitFailed(hWnd, hRet,
	                "Blt() on background failed");
		    return;
	 }
	 
	 // Flip surfaces
     hRet = lpDDSPrimary->Flip(NULL, DDFLIP_WAIT);
     if(hRet != DD_OK){
        DDInitFailed(hWnd, hRet,
	                "Flip() call failed");
		      return;
		}
	return;
   }
}


//***************************************
// Name: DDInitFailed()
// Desc: This function is called if an
//       initialization call fails
//***************************************
HRESULT DDInitFailed(HWND hWnd, HRESULT hRet, LPCTSTR szError)
{
    char   szBuff[128];

    sprintf(szBuff, szError);
    ReleaseObjects();
    MessageBox(hWnd, szBuff, "DirectDraw Demo No. 2", MB_OK);
    DestroyWindow(hWnd);
    return hRet;

}

//******************************************
// Name: DD4Interface()
// Desc: Finds DirectDraw4 object
// 
// POST CONDITIONS:
//   returns 0 if no DirectDraw4 interface
//   returns 1 if DirectDraw4 found
//******************************************
int DD4Interface()
{

	HRESULT         DDConnect;
	LPDIRECTDRAW	lpDD;    // Pointer to DirectDraw

	DDConnect = DirectDrawCreate ( NULL,
	           &lpDD,
			   NULL);
     
	if(DDConnect != DD_OK)
		return 0;
    // Atempt to locate DirectDraw4 interface
    DDConnect = lpDD->QueryInterface(
                IID_IDirectDraw4,
	            (LPVOID *) &lpDD4);
    if(DDConnect != S_OK)
		return 0;

    lpDD->Release();      // Release old pointer
        return 1;

}


//***********************************************
// Name: ModesProc
// Desc: Callback function for EnumDisplayModes()
//***********************************************
HRESULT WINAPI ModesProc(LPDDSURFACEDESC2 aSurface,
								  LPVOID Context)
{
   static int i;         // Index into array of mode data

   i = modesCount * 3;   // Set array pointer

   if( modesCount >= MAX_MODES )
	   return DDENUMRET_CANCEL;  // Stop mode listing

   // Store mode paramenters in public array
   // Note: code assumes that the dwRGBBitFormat member if
   //       the DDPIXELFORMAT structure contains valid data
   modesArray[i]     = aSurface->dwWidth;
   modesArray[i + 1] = aSurface->dwHeight;
   modesArray[i + 2] = aSurface->ddpfPixelFormat.dwRGBBitCount;

   modesCount++;         // Bump display modes counter
   return DDENUMRET_OK;  // Continue

}


//***************************************************************
// Name: hasDDMode
// Desc: Tests for mode availability
//***************************************************************
// PRE CONDITIONS:
// 1. Public array modesArray[] to store mode data
// 2. Public int variable modesCount to store number of
//    display modes
// 3. ModesProc() is an associated function that stores
//    mode data in array and count modes
//
// POST CONDITIONS:
//  1. Returns 1 if mode is available
//***************************************************************

int HasDDMode(DWORD pixWidth, DWORD pixHeight, DWORD pixBits)
{

	static HRESULT DDConnect;

    // Call EnumDisplayModes() 
	if(MODES_ON == 0){
		MODES_ON = 1;         // set switch
		DDConnect = lpDD4->EnumDisplayModes(0, NULL, NULL,
			                                ModesProc);
	}
// Modes are now stored in modeArray[] as triplets encoding
// width, height, and pixel bit size
// Variable modesCount holds the total number of display modes
	for(int x = 0; x < (modesCount * 3); x += 3){
		if(modesArray[x]==pixWidth && modesArray[x+1]==pixHeight\
			&& modesArray[x+2]==pixBits)
			return 1;
	}
	return 0;
}


//***************************************************************
// Name: LockSurface
// Desc: Function to lock a DirectDraw surface and store the
//       direct access parameters
// PRE CONDITIONS:
// 1. Parameter is pointer to DirectDraw surface
// 2. Video display globals have been declared as follows:
// 	  LONG			vidPitch;	// Pitch
//	  LPVOID		vidStart;	// Buffer address
//
// POST CONDITIONS:
// Returns 1 if call succeeds and 0 if it failed
//***************************************************************
int LockSurface(LPDIRECTDRAWSURFACE4  lpSurface) 
{
	// Attempt to lock the surface for direct access
	if (lpSurface->Lock(NULL, &ddsd, DDLOCK_WAIT, NULL)\
		 != DD_OK)
		return 0;                  // Lock failed
	// Store video system data
    vidPitch = ddsd.lPitch;        // Pitch
	vidStart = ddsd.lpSurface;     // Buffer address	
    return 1;					   // Surface locked   
}

//***************************************************************
// Name: ReleaseSurface
// Desc: Function to release locked surface 
// PRE CONDITIONS:
// 1. Parameter is pointer to DirectDraw surface
// 2. Video display globals as follows:
// 	  LONG			vidPitch;	// Pitch
//	  LPVOID		vidStart;	// Buffer address
//***************************************************************
void ReleaseSurface(LPDIRECTDRAWSURFACE4  lpSurface) 
{
	lpSurface->Unlock(NULL);
	// Clear global variables
	vidPitch = 0;
	vidStart = 0;
    return;
}



//***************************************************************
// Name: DDBmapToSurf
// Desc: Moves a bitmap to a DirectDraw Surface
// PRE CONDITIONS:
// 1. Parameter 1 is pointer to a DirectDraw surface
//    Parameter 2 is handle to the bitmap
//
// POST CONDITIONS:
// Bitmap is move to surface
// Returns 1 if successful
// /
// ERROR CONDITIONS:
// All errors exit thorugh DDInitFailed() function
//***************************************************************
HRESULT DDBmapToSurf(LPDIRECTDRAWSURFACE4 pdds, HBITMAP hbm)
{
    HDC                     hdcImage;
    HDC                     hdc;
    DDSURFACEDESC2          ddsd;
    HRESULT                 hr = 1;
	BOOL					retValue;

    if (hbm == NULL || pdds == NULL)
 		DDInitFailed(hWnd, hRet,
		       "Invalid surface or bitmap in DDBmapToSurf");

   // Create compatible DC and select bitmap into it
	hdcImage = CreateCompatibleDC(NULL);
    if (!hdcImage)
 		DDInitFailed(hWnd, hRet,
		       "CreateCompatibleDC() failed in DDBmapToSurf");
    SelectObject(hdcImage, hbm);
 
    // Get size of surface
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    pdds->GetSurfaceDesc(&ddsd);

    if ((hr = pdds->GetDC(&hdc)) != DD_OK)
 		DDInitFailed(hWnd, hRet,
		       "GetDC() failed in DDBmapToSurf");

	 retValue = BitBlt(hdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight,
		               hdcImage, 0, 0, SRCCOPY);
	 // Release surface immediately
	 pdds->ReleaseDC(hdc);

	 if(retValue == FALSE)
   		DDInitFailed(hWnd, hRet,
		       "BitBlt() failed in DDBmapToSurf");

        DeleteDC(hdcImage);
    return hr;
}


