//*************************************************************
//*************************************************************
// 3DRM InWin Demo1
//
// Chapter Reference: 15
//
// Notes:
//       First Demo program for retained mode
// Description:
//             Static display of a file in DirectX format using
// Direct3D retained mode.
//*************************************************************
// Libraries (referenced at link-time):
//          dxguid.lib
//          dddraw.lib
//          winmm.lib   
//          d3drm.lib
//*************************************************************
//*************************************************************
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN  // Exclude unnecessary components
#endif

#define INITGUID		// Required for Direct3D
#include <windows.h>	// Windows header files
#include <ddraw.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdarg.h>
#include <d3drmwin.h>	// Direct3D retained mode definitions
#include "resource.h"


// Global variables
struct _globVars
{
    LPDIRECT3DRMDEVICE3 aDevice;	// Retained mode aDevice          
    LPDIRECT3DRMVIEWPORT2 aViewport;// Direct3DRM viewport through which
								    // to aViewport the aScene. 
    LPDIRECT3DRMFRAME3 aScene;      // Master frame    
    LPDIRECT3DRMFRAME3 aCamera;     // Frame for user's point of view  
    BOOL isInitialized;		    	// All D3DRM objects have been initialized.         
} globVars;

LPDIRECT3DRM3 lpD3DRM = NULL;            // Direct3D RM object manager
LPDIRECTDRAWCLIPPER lpDDClipper = NULL;  // DirectDrawClipper object 
HWND			hWnd;
char			szXfile[] = "teapot.x" ;  // File to load


// Function prototypes 
BOOL BuildScene( LPDIRECT3DRMDEVICE3, 
                 LPDIRECT3DRMFRAME3, LPDIRECT3DRMFRAME3 );
static BOOL RenderScene();
BOOL CreateObjects( HWND hwnd );
BOOL InitD3D(HWND);
void D3DError(LPCTSTR);

LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );

//*****************************************************************
// WinMain implementation for Direct3D retained mode execution
//*****************************************************************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{

    MSG             msg;

    static char szAppName[] = "DDAppName" ; // Application name


     // Defining a structure of type WNDCLASSEX
     WNDCLASSEX  wndclass ;
     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (hInstance,
		                                MAKEINTRESOURCE(IDI_ICON1));
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject
                              (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName;
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon (hInstance,
		                                MAKEINTRESOURCE(IDI_ICON1));
	 
	 // Register the class
	 RegisterClassEx(&wndclass);


              
     hWnd = CreateWindowEx(0,        // Extended style
                szAppName,
                "3DRM InWin Demo1",
                WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				350,			     // Initial screen size
				300,
				NULL,            // Handle of parent
                NULL,            // Handle to menu
                hInstance,       // Application instance
                NULL);           // Additional data

    if (!hWnd)
        return FALSE;



    // Display the window.
    ShowWindow(hWnd, nCmdShow);

	while (GetMessage(&msg, NULL, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }


    return msg.wParam;
}

//*****************************************************************
// Name: WndProc
// Desc: Window procedure
//*****************************************************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam,
LPARAM lParam)
{
    switch (message) 
    {

	case WM_CREATE:
	    // Initialize global variables and create the Direct3D Retained
		// Mode objects.
		if(!InitD3D(hwnd)){
			PostMessage(NULL, WM_DESTROY, 0, 0);
			break;
		}
		if (!CreateObjects(hwnd)){
			PostMessage(NULL, WM_DESTROY, 0, 0);
			break;
		}
		case WM_PAINT:
        // If the application is not minimized and the Direct3D Retained 
        // Mode objects are initialized, render.
        if (globVars.isInitialized) 
        {
			// Render one frame of the aScene. If rendering fails, post
            // the WM_DESTROY message, enabling the application to 
            // free resources before terminating the application.
            if (!RenderScene()) 
            {
                D3DError("Rendering failed. Aborting execution.");
                PostMessage(NULL, WM_DESTROY, 0, 0);
                break;
            }
		// Yield to other applications if this application is
        // not currently rendering.
		}
		break;

        case WM_DESTROY:
            // Clear the isInitialized flag, free objects, and post the 
            // WM_QUIT message to terminate the application. Be sure to
            // free the viewport before freeing the device or a general 
            // protection fault might occur.
			globVars.isInitialized = FALSE;
            globVars.aViewport->Release();
            globVars.aDevice->Release();
            globVars.aCamera->Release();
            globVars.aScene->Release();
            lpD3DRM->Release();
            lpDDClipper->Release();
			PostQuitMessage(0);
            break;

        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }
    return 0L;
}


//***************************************************************
// Name: BuildScene()
// Description: Create the scene
//***************************************************************
BOOL BuildScene( LPDIRECT3DRMDEVICE3 aDevice, 
                 LPDIRECT3DRMFRAME3  aScene, 
                 LPDIRECT3DRMFRAME3  aCamera )
{
    // Local varaibles
    LPDIRECT3DRMFRAME3 lights            = NULL;
    LPDIRECT3DRMMESHBUILDER3 meshbuilder = NULL;
    LPDIRECT3DRMFRAME3 childframe        = NULL;
    LPDIRECT3DRMLIGHT light1             = NULL;
    LPDIRECT3DRMLIGHT light2             = NULL;
    LPDIRECT3DRMMATERIAL2 material1      = NULL;
    HRESULT retval;

    // Create the meshbuilder object
    retval = lpD3DRM->CreateMeshBuilder(&meshbuilder);
      if (FAILED(retval))
          goto ERROR_EXIT;

    // Use meshbuilder to load a mesh from a DirectX file
    retval = meshbuilder->Load(szXfile,            // Source
                               NULL,
                               D3DRMLOAD_FROMFILE, // Options
                               NULL, NULL);
    if (FAILED(retval)) 
    {
        D3DError("Failed to load file.");
        goto DIRECT_EXIT;
    }

    // Create a child frame within the aScene.
    retval = lpD3DRM->CreateFrame(aScene, &childframe);
    if(FAILED(retval))
        goto ERROR_EXIT;

    // Add mesh into the child frame as a visual
    retval = childframe->AddVisual(
                               (LPDIRECT3DRMVISUAL)meshbuilder);
    if(FAILED(retval))
        goto ERROR_EXIT;
    // Set up the camera frame position
    retval = aCamera->SetPosition(aScene,
                                  D3DVAL(0),      // x
                                  D3DVAL(0),      // y
                                  -D3DVAL(7));    // z
    if (FAILED(retval)) 
    {
        D3DError("Failed to position the camera in the frame.");
        goto DIRECT_EXIT;
    }
    // Create a light frame as a child of the scene frame
    retval = lpD3DRM->CreateFrame(aScene, &lights);
      if(FAILED(retval))
        goto ERROR_EXIT;

    // Position the light frame within the scene
    retval = lights->SetPosition(aScene,
                                 D3DVAL(5),       // x
                                 D3DVAL(0),       // y
                                 -D3DVAL(7));     // z
    if(FAILED(retval))
        goto ERROR_EXIT;

    // Create a bright, parallel point light
    // Color values are as follows:
    // 0.0 = totally dim and 1.0 = totally bright
    retval = lpD3DRM->CreateLightRGB(D3DRMLIGHT_PARALLELPOINT,  
                             D3DVAL(0.8),   // Red intensity
                             D3DVAL(0.8),   // Green intensity
                             D3DVAL(1.0),   // Blue intensity
                             &light1);
    if(FAILED(retval))
        goto ERROR_EXIT;

    // Add light to light frame
    retval = lights->AddLight(light1);
    if(FAILED(retval))
        goto ERROR_EXIT;

    // Create a dim, ambient light and attach it to the scene 
    // frame,
    retval = lpD3DRM->CreateLightRGB(D3DRMLIGHT_AMBIENT, 
                                       D3DVAL(0.2),    // red
                                       D3DVAL(0.2),    // green
                                       D3DVAL(0.2),    // blue
                                       &light2);
    if(FAILED(retval))
        goto ERROR_EXIT;
  
    retval = aScene->AddLight(light2);
       if(FAILED(retval))
            goto ERROR_EXIT;

    // Create a material setting its specular property
    retval = lpD3DRM->CreateMaterial(D3DVAL(8.0), &material1);
    if(FAILED(retval))
        goto ERROR_EXIT;

    // Set the material on the mesh
    retval = meshbuilder->SetMaterial(material1);
    if(FAILED(retval))
        goto ERROR_EXIT;

    // Set the mesh color (bright green in this case). 
    retval = meshbuilder->SetColorRGB(D3DVAL(0.0),   // red 
                                     D3DVAL(0.7),    // green 
                                     D3DVAL(0.0));   // blue

    if(FAILED(retval))
        goto ERROR_EXIT;

    //******************************
    // Function succeeds. Clean up
    //******************************  
    childframe->Release();
    lights->Release();
    meshbuilder->Release();
    light1->Release();
    light2->Release();
    material1->Release();
    return TRUE;

    //******************************
    //       Error exits
    //******************************  
    ERROR_EXIT:
    D3DError("Failure building the scene");
    DIRECT_EXIT:
    childframe->Release();
    lights->Release();
    meshbuilder->Release();
    light1->Release();
    light2->Release();
    material1->Release();    
    return FALSE;
}

//**************************************************************
// Name: RenderScene()
// Description: Clear the viewport, render the frame, and
// update the window.
//**************************************************************
static BOOL RenderScene()
{
    HRESULT retval;

    // Clear the viewport.
    retval = globVars.aViewport->Clear(D3DRMCLEAR_ALL);
    if (FAILED(retval)) 
    {
        D3DError("Clearing viewport failed.");
        return FALSE;
    }

    // Render the aScene to the viewport.
    retval = globVars.aViewport->Render(globVars.aScene);
    if (FAILED(retval)) 
    {
        D3DError("Rendering scene failed.");
        return FALSE;
    }
    // Update the window.
    retval = globVars.aDevice->Update();
    if (FAILED(retval)) 
    {
        D3DError("Updating device failed.");
        return FALSE;
    }
    return TRUE;
}

//********************************************************
// Name: InitD3D()
// Descriotion: Initialize Direct3D interface 
//********************************************************
BOOL InitD3D(HWND hwnd)
{
    HRESULT retval; // Return value

    // Initialize the entire global variable structure to zero. 
    memset(&globVars, 0, sizeof(globVars));

    // Create the Direct3DRM object.
    LPDIRECT3DRM pD3DRMTemp;
    retval = Direct3DRMCreate(&pD3DRMTemp);
    if (FAILED(retval)) 
    {
        D3DError("Failed to create Direct3DRM.");
        return FALSE;
    }
    if( FAILED( pD3DRMTemp->QueryInterface(IID_IDirect3DRM3,
                            (void **)&lpD3DRM) ) )
    {
        pD3DRMTemp->Release();
        D3DError("Direct3DRM3 interface not found" );
        return FALSE;
    }
    pD3DRMTemp->Release();

    // Create DirectDrawClipper object
    retval = DirectDrawCreateClipper(0, &lpDDClipper, NULL);
    if (FAILED(retval)) 
    {
        D3DError("Failed to create DirectDrawClipper object");
        return FALSE;
    }
    // Attach the program Window as a clipper
    retval = lpDDClipper->SetHWnd(0, hwnd);
    if (FAILED(retval)) 
    {
        D3DError("Failed to set the window handle");
        return FALSE;
    }
return TRUE;
}

//***************************************************************
// Name: CreateObjects()
// Description: Create the device and the scene objects
//***************************************************************
BOOL CreateObjects( HWND hwnd )
{
    // Local variables
    HRESULT retval;  // Return value 
    RECT rc;         // Bounding rectangle for main window 
    int width;       // Device's width 
    int height;      // Device's height 
    
    // Get client area dimensions
    GetClientRect(hwnd, &rc);
    // Create device from DirectDraw clipper
    retval = lpD3DRM->CreateDeviceFromClipper(lpDDClipper, 
                           NULL,               // Default device 
                           rc.right, rc.bottom,
                           &globVars.aDevice);
    if (FAILED(retval)) 
    {
        D3DError("Failed to create the D3DRM device");
        return FALSE;
    }

    // Create the master scene
    retval = lpD3DRM->CreateFrame(NULL, &globVars.aScene);
    if (FAILED(retval)) 
    {
        D3DError("Failed to create the master scene frame");
        return FALSE;
    }
	// Create the camera frame
    retval = lpD3DRM->CreateFrame(globVars.aScene,
                                  &globVars.aCamera);
    if (FAILED(retval)) 
    {
        D3DError("Failed to create the camera frame");
        return FALSE;
    }

    // Create the Direct3DRM viewport using the device, the
    // camera frame, and the device's width and height.
    width = globVars.aDevice->GetWidth();
    height = globVars.aDevice->GetHeight();

    retval = lpD3DRM->CreateViewport(globVars.aDevice,
                                     globVars.aCamera, 0, 0,
                                     width, height, 
                                     &globVars.aViewport);
    if (FAILED(retval)) 
    {
        globVars.isInitialized = FALSE;
        globVars.aDevice->Release();
        return FALSE;
    }
    // Create the scene
    if (!BuildScene(globVars.aDevice, globVars.aScene, 
                    globVars.aCamera))
        return FALSE;

    // Record that global variables are initialized
    globVars.isInitialized = TRUE;
    return TRUE;
}


//***************************************
// Name: D3DD3DError()
// Desc: This function is called if a
//       D3D critical function fails
//***************************************
void D3DError(LPCTSTR szError)
{
    char   szBuff[128];

    sprintf(szBuff, szError);
    MessageBox(NULL, szBuff, "Direct3D in Win Demo 1", MB_OK);
    PostMessage(NULL, WM_DESTROY, 0, 0);
    return;
}



